import requests
from sephiroth.providers.base_provider import BaseProvider


class OCI(BaseProvider):
    def __init__(self, excludeip6=False):
        self.source_ranges = self._get_ranges()
        self.processed_ranges = self._process_ranges(excludeip6)

    def _get_ranges(self):
        """
        Input: None
        Output: Dict representation of ip-ranges.json
        """
        print("(oci) Fetching IP ranges from Oracle")
        oci_ip_ranges_url = (
            "https://docs.cloud.oracle.com/en-us/iaas/tools/public_ip_ranges.json"
        )
        r = requests.get(oci_ip_ranges_url)
        return r.json()

    def _process_ranges(self, excludeip6=False):
        """
        Input: Dict of ip-ranges.json, optionally exclude ip6 ranges
        Output: Dict with header_comments and list of dicts for ip ranges
        """
        header_comments = [
            f"(oci) last_updated_timestamp: {self.source_ranges['last_updated_timestamp']}"
        ]
        out_ranges = []

        for r in self.source_ranges["regions"]:
            region = r["region"]
            for cidr in r["cidrs"]:
                item = {
                    "range": cidr["cidr"],
                    "comment": f"{region} {' '.join(cidr['tags'])}",
                }
                out_ranges.append(item)

        return {"header_comments": header_comments, "ranges": out_ranges}



import requests
from sephiroth.providers.base_provider import BaseProvider


class Tor(BaseProvider):
    def __init__(self, excludeip6=False):
        self.source_ranges = self._get_ranges()
        self.processed_ranges = self._process_ranges()

    def _get_ranges(self):
        """
        Input: None
        Output: List of ip addresses
        """
        print("(tor) Fetching Tor exit nodes from torproject.org")
        exit_list_url = "https://check.torproject.org/torbulkexitlist"
        r = requests.get(exit_list_url)
        return r.content.decode("utf-8").split("\n")

    def _process_ranges(self):
        """
        Input: Dict of ip-ranges.json, optionally exclude ip6 ranges
        Output: Dict with header_comments and list of dicts for ip ranges
        """
        header_comments = ["(tor) Exit nodes collected from from check.torproject.org"]
        out_ranges = []
        for address in self.source_ranges:
            if not address:
                continue
            item = {"range": address, "comment": "tor exit node"}
            out_ranges.append(item)
        return {"header_comments": header_comments, "ranges": out_ranges}



class BaseProvider:
    def _get_ranges(self):
        raise NotImplementedError

    def _process_ranges(self):
        raise NotImplementedError

    def get_processed_ranges(self):
        return self.processed_ranges



from csv import DictReader
import io
import requests

from sephiroth.providers.base_provider import BaseProvider


class Linode(BaseProvider):
    def __init__(self):
        self.source_ranges = self._get_ranges()
        self.processed_ranges = self._process_ranges()

    def _get_ranges(self):
        """
        Input: None
        Output: Dict representation of geoip.linode.com
        """
        print("(linode) Fetching Linode addresses from geoip.linode.com")
        fieldnames = ("prefix", "country", "subdivision", "city", "allocation_size")
        url = "https://geoip.linode.com/"
        r = requests.get(url)
        r.encoding = "utf-8"
        csvio = io.StringIO(r.text, newline="")
        return list(DictReader(csvio, fieldnames))

    def _process_ranges(self):
        """
        Input: Dict of ip-ranges.json, optionally exclude ip6 ranges
        Output: Dict with header_comments and list of dicts for ip ranges
        """
        header_comments = ["(linode) Address nodes collected from geoip.linode.com"]
        out_ranges = []
        for address in self.source_ranges:
            if not address:
                continue
            if address["prefix"].startswith("# Last modified"):
                header_comments.append(f"(linode) {address['prefix'][2:]}")
                continue
            elif address["prefix"].startswith("#"):
                continue
            item = {
                "range": address["prefix"],
                "comment": f"{address['country']} {address['subdivision']} {address['city']}",
            }
            out_ranges.append(item)
        return {"header_comments": header_comments, "ranges": out_ranges}



from sephiroth.providers.aws import AWS
from sephiroth.providers.azure import Azure
from sephiroth.providers.gcp import GCP
from sephiroth.providers.oci import OCI
from sephiroth.providers.asn import ASN
from sephiroth.providers.file import File
from sephiroth.providers.tor import Tor
from sephiroth.providers.do import DO
from sephiroth.providers.linode import Linode
from sephiroth.providers.cloudflare import Cloudflare

# This must be imported last, circular dependencies and all that
from sephiroth.providers.provider import Provider

__all__ = [
    "AWS",
    "Azure",
    "GCP",
    "OCI",
    "ASN",
    "File",
    "Tor",
    "DO",
    "Linode",
    "Provider",
    "Cloudflare",
]



import requests
from sephiroth.providers.base_provider import BaseProvider


class ASN(BaseProvider):
    def __init__(self, targets_in, excludeip6=False):
        self.source_ranges = self._get_ranges(targets_in)
        self.processed_ranges = self._process_ranges(excludeip6)

    def _get_ranges(self, asns):
        """
        Input: List of ASNs in AS#### format
        Output: Dict representation of ip-ranges.json
        """
        ranges = {}
        print(
            f"(asn) Fetching IP ranges from api.hackertarget.com for {len(asns)} ASNs"
        )
        for a in asns:
            asn_lookup_url = f"https://api.hackertarget.com/aslookup/?q={a}"
            r = requests.get(asn_lookup_url)
            # the first result from this API is always the ASN name and number
            ranges[a] = r.content.decode("utf-8").split("\n")[1:]
        return ranges

    def _process_ranges(self, excludeip6=False):
        """
        Input: Dict of ip-ranges.json, optionally exclude ip6 ranges
        Output: Dict with header_comments and list of dicts for ip ranges
        """
        header_comments = ["(asn) ASN Data collected from api.hackertarget.com"]
        out_ranges = []
        for asn, range_list in self.source_ranges.items():
            for ip_range in range_list:
                if ":" in ip_range and excludeip6:
                    continue
                item = {"range": ip_range, "comment": f"{asn}"}
                out_ranges.append(item)
        return {"header_comments": header_comments, "ranges": out_ranges}



import requests
from bs4 import BeautifulSoup

from sephiroth.providers.base_provider import BaseProvider


class Azure(BaseProvider):
    def __init__(self, excludeip6=False):
        self.source_ranges = self._get_ranges()
        self.processed_ranges = self._process_ranges(excludeip6)

    def _get_ranges(self):
        print("(azure) Fetching IP ranges from Microsoft")
        azure_download_page = (
            "https://www.microsoft.com/en-us/download/details.aspx?id=56519"
        )
        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:109.0) Gecko/20100101 Firefox/112.0'
        }
        r = requests.get(azure_download_page, headers = headers)
        soup = BeautifulSoup(r.content, "html.parser")
        direct_link = soup.select_one(".dlcdetail__download-btn")["href"]
        r = requests.get(direct_link)
        return r.json()

    def _process_ranges(self, excludeip6=False):
        header_comments = [
            f"(azure) changeNumber: {self.source_ranges['changeNumber']}",
            f"(azure) cloud: {self.source_ranges['cloud']}",
        ]
        out_ranges = []
        for item in self.source_ranges["values"]:
            name = item["name"]
            platform = item["properties"]["platform"]
            for addr in item["properties"]["addressPrefixes"]:
                out_item = {"range": addr, "comment": f"{platform} {name}"}
                out_ranges.append(out_item)

        return {"header_comments": header_comments, "ranges": out_ranges}



import requests
from sephiroth.providers.base_provider import BaseProvider


class AWS(BaseProvider):
    def __init__(self, excludeip6=False):
        self.source_ranges = self._get_ranges()
        self.processed_ranges = self._process_ranges(excludeip6)

    def _get_ranges(self):
        """
        Input: None
        Output: Dict representation of ip-ranges.json
        """
        print("(aws) Fetching IP ranges from Amazon")
        aws_ip_ranges_url = "https://ip-ranges.amazonaws.com/ip-ranges.json"
        r = requests.get(aws_ip_ranges_url)
        return r.json()

    def _process_ranges(self, excludeip6=False):
        """
        Input: Dict of ip-ranges.json, optionally exclude ip6 ranges
        Output: Dict with header_comments and list of dicts for ip ranges
        """
        header_comments = [
            f"(aws) syncToken: {self.source_ranges['syncToken']}",
            f"(aws) createDate: {self.source_ranges['createDate']}",
        ]
        out_ranges = []
        source_prefixes = self.source_ranges["prefixes"]
        if not excludeip6:
            source_prefixes += self.source_ranges["ipv6_prefixes"]

        for prefix in source_prefixes:
            if "ipv6_prefix" in prefix:
                item_prefix = prefix["ipv6_prefix"]
                iptype = "ipv6"
            else:
                item_prefix = prefix["ip_prefix"]
                iptype = "ipv4"
            item = {
                "range": item_prefix,
                "comment": f"{iptype} {prefix['region']} {prefix['service']}",
            }
            out_ranges.append(item)

        return {"header_comments": header_comments, "ranges": out_ranges}



import requests
from sephiroth.providers.base_provider import BaseProvider


class Cloudflare(BaseProvider):
    def __init__(self, excludeip6=False):
        self.source_ranges = self._get_ranges()
        self.processed_ranges = self._process_ranges(excludeip6)

    def _get_ranges(self):
        """
        Input: None
        Output: Dict representation of cloudflare ip ranges
        """
        print("(cloudflare) Fetching IP ranges from cloudflare's API")
        cloudflare_ip_ranges_url = "https://api.cloudflare.com/client/v4/ips"
        r = requests.get(cloudflare_ip_ranges_url)
        return r.json()

    def _process_ranges(self, excludeip6=False):
        """
        Input: List of ip-ranges, optionally exclude ip6 ranges
        Output: Dict with header_comments and list of dicts for ip ranges
        """
        header_comments = [
            f"(cloudflare) success: {self.source_ranges['success']}",
            f"(cloudflare) messages: {self.source_ranges['messages']}",
        ]
        out_ranges = []
        ip_ranges = self.source_ranges["result"]

        range_types = [("ipv4_cidrs", "ipv4 cloudflare")]

        if not excludeip6:
            range_types.append(("ipv6_cidrs", "ipv6 cloudflare"))

        for range_type, comment in range_types:
            for ip_range in ip_ranges[range_type]:
                item = {
                    "range": ip_range,
                    "comment": comment
                }
                out_ranges.append(item)

        return {"header_comments": header_comments, "ranges": out_ranges}



import requests
import io
from csv import DictReader

from sephiroth.providers.base_provider import BaseProvider


class DO(BaseProvider):
    def __init__(self, excludeip6=False):
        self.source_ranges = self._get_ranges()
        self.processed_ranges = self._process_ranges(excludeip6)

    def _get_ranges(self):
        """
        Input: None
        Output: Dict representation of google.csv from DO
        """
        print("(do) Fetching IP ranges from Digital Ocean")

        # The URL for this CSV is published on the DO Platform page:
        # https://www.digitalocean.com/docs/platform/
        # They indicate it is updated automatically, but they don't publish a
        # definition of the headers or policy regarding its' update or use.
        do_ip_ranges_url = "https://digitalocean.com/geo/google.csv"
        fieldnames = ("prefix", "country", "region", "city", "postalcode")

        r = requests.get(do_ip_ranges_url)
        r.encoding = "utf-8"
        csvio = io.StringIO(r.text, newline="")
        return list(DictReader(csvio, fieldnames))

    def _process_ranges(self, excludeip6=False):
        """
        Input: Dict of google.csv, optionally exclude ip6 ranges
        Output: Dict with header_comments and list of dicts for ip ranges
        """
        header_comments = [
            "(do) IP ranges collected from https://digitalocean.com/geo/google.csv"
        ]

        out_ranges = []
        source_prefixes = self.source_ranges

        for prefix in source_prefixes:
            if ":" in prefix["prefix"]:
                iptype = "ipv6"
                if excludeip6:
                    continue
            else:
                iptype = "ipv4"

            item_prefix = prefix["prefix"]

            item = {
                "range": item_prefix,
                "comment": f"{iptype} {prefix['region']} {prefix['city']}",
            }
            out_ranges.append(item)

        return {"header_comments": header_comments, "ranges": out_ranges}



import setuptools
import sephiroth

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="sephiroth",
    version=sephiroth.__version__,
    author=sephiroth.__author__,
    author_email=sephiroth.__email__,
    maintainer=sephiroth.__maintainer__,
    description="Utility for building blocklists for ip ranges",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/0xdade/sephiroth",
    packages=["sephiroth", "sephiroth.providers"],
    install_requires=[
        "beautifulsoup4 ~= 4.11.1",
        "requests ~= 2.27.1",
        "Jinja2 ~= 3.1.1",
        "netaddr~=0.8.0",
    ],
    py_modules=["Sephiroth"],
    include_package_data=True,
    entry_points={"console_scripts": ["sephiroth = Sephiroth:main"]},
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12"
    ],
    python_requires=">=3.6",
)



from sephiroth.providers import AWS, Azure, GCP, OCI, ASN, File, Tor, DO, Linode, Cloudflare

classmap = {
    "aws": AWS,
    "azure": Azure,
    "gcp": GCP,
    "oci": OCI,
    "asn": ASN,
    "file": File,
    "tor": Tor,
    "do": DO,
    "linode": Linode,
    "cloudflare": Cloudflare,
}


class Provider:
    def __init__(self, provider, targets_in=None):
        if targets_in:
            self.provider = classmap[provider](targets_in)
        else:
            self.provider = classmap[provider]()

    def get_processed_ranges(self):
        return self.provider.get_processed_ranges()

    def get_compacted_ranges(self):
        from netaddr import IPNetwork, IPSet

        processed = self.provider.get_processed_ranges()
        networks = [IPNetwork(cidr["range"]) for cidr in processed["ranges"]]
        compacted = [
            {
                "range": str(cidr),
                "comment": f"{type(self.provider).__name__.lower()} (compacted)",
            }
            for cidr in IPSet(networks).iter_cidrs()
        ]

        return {"header_comments": processed["header_comments"], "ranges": compacted}



#!/usr/bin/env python3

import argparse
from datetime import datetime
import os
from datetime import timezone
from pathlib import Path

from jinja2 import Template

from sephiroth.providers import Provider
from sephiroth.providers.provider import classmap as supported_targets
import sephiroth

supported_servers = ["nginx", "apache", "iptables", "ip6tables"]

base_dir = os.path.dirname(__file__)
template_dir = os.path.join(base_dir, "sephiroth", "templates")
output_dir = os.path.join(os.getcwd(), "output")


def get_output_path(servertype, targets, build_date):
    """
    Input: Server type, date from build_template(), cloud provider
    Output: Path to file on disk to write to
    """
    targets_str = "_".join(targets)
    fdate = build_date.strftime("%Y-%m-%d_%H%M%S")
    fname = f"{fdate}_{servertype}_{targets_str}.conf"
    return os.path.join(output_dir, fname)


def get_ranges(selected_provider, excludeip6=False, targets_in=None, compacted=False):
    """
    Input: Type of provider to target, as defined in supported_targets.
           Optionally exclude ip6, provide list of asns or files if asn or file target
    Output: Structured data ready to go to templates
    """
    if targets_in:
        provider = Provider(selected_provider, targets_in)
    else:
        provider = Provider(selected_provider)
    if compacted:
        return provider.get_compacted_ranges()
    return provider.get_processed_ranges()


def get_template(servertype):
    """
    Input: String name of server type
    Output: Jinja2 template object for given server type
    """
    fname = f"{servertype}/conf.jinja"
    template_path = os.path.join(template_dir, fname)
    return Template(open(template_path).read())


def build_template(ranges, template, build_date, use_proxy=False, redir_target=""):
    """
    Input: output of process_<provider>_ranges(), output of get_template()
    Output: Rendered template string ready to write to disk
    """
    return template.render(
        ranges=ranges["ranges"],
        header_comments=ranges["header_comments"],
        build_date=build_date,
        use_proxy=use_proxy,
        redir_target=redir_target,
    )


def print_output(servertype, targets, outfile):
    helpfile = os.path.join(template_dir, servertype, "help.jinja")
    abspath = os.path.abspath(outfile)
    targets_str = ", ".join(targets)
    help_text = Template(open(helpfile).read()).render(
        abspath=abspath, outfile=os.path.basename(outfile)
    )
    print(f"Your {servertype} blocklist for {targets_str} can be found at {outfile}\n")
    print(help_text)


def validate_nginx_args(args):
    if args.redir_target:
        print(
            "[?] Warning: We cannot generate nginx configs with redirect targets at this time. Ignoring."
        )
    return True


def validate_apache_args(args):
    if args.use_proxy:
        print(
            "[?] Warning: We cannot use PROXY protocol with Apache at this time. Ignoring."
        )
    if args.redir_target is None:
        print("[!] Error: Apache requires a defined redirect target using -r")
        raise SystemExit
    elif args.redir_target.startswith("http://") or args.redir_target.startswith(
        "https://"
    ):
        print(
            "[!] Error: Redirect target should not include scheme. Please edit the output RewriteRule directly if you want to change this."
        )
        raise SystemExit
    return True


def validate_iptables_args(args):
    print("[?] Warning: iptables rules automatically exclude any IPv6 addresses")
    return True


def validate_ip6tables_args(args):
    print("[?] Warning: ip6tables rules automatically exclude any IPv4 addresses")
    return True


def parse_args():
    parser_desc = "Sephiroth is made to help block clouds."
    parser_epilog = "For more information, assistance, or to submit a pull request, please visit https://github.com/0xdade/sephiroth."
    parser = argparse.ArgumentParser(
        prog="Sephiroth", description=parser_desc, epilog=parser_epilog
    )
    parser.add_argument(
        "-s",
        "--server",
        help="Type of server to build blocklist for",
        required=True,
        choices=supported_servers,
        dest="servertype",
    )
    parser.add_argument(
        "-t",
        "--target",
        help="Targets to block",
        required=True,
        choices=supported_targets,
        action="append",
        dest="targets",
    )
    parser.add_argument(
        "-a",
        "--asn",
        help="ASN to block in AS#### format",
        action="append",
        metavar="ASN",
        dest="asns",
    )
    parser.add_argument(
        "-f",
        "--file",
        help="Files to block addresses from",
        action="append",
        metavar="FILENAME",
        dest="files",
    )
    parser.add_argument(
        "-r",
        "--redir",
        help="Place to redirect requests to. (apache)",
        default=None,
        dest="redir_target",
    )
    parser.add_argument(
        "-p",
        "--proxy",
        help="Using PROXY Protocol? (nginx)",
        default=False,
        action="store_true",
        dest="use_proxy",
    )
    parser.add_argument(
        "--no-ipv6",
        help="Exclude ipv6 addresses from the block list where applicable",
        default=False,
        action="store_true",
        dest="excludeip6",
    )
    parser.add_argument(
        "--compacted",
        help="Compact neighboring cidr ranges. This produces smaller file sizes but loses detail about each range.",
        default=False,
        action="store_true",
        dest="compacted",
    )
    parser.add_argument(
        "-V", "--version", action="version", version="%(prog)s " + sephiroth.__version__
    )
    return parser.parse_args()


def validate_targets(args):
    success = True
    for target in args.targets:
        if target == "asn" and not args.asns:
            print(
                "[!] Error: Cannot specify -t asn without including at least one -a AS####"
            )
            success = False
        elif target == "file" and not args.files:
            print(
                "[!] Error: Cannot specify -t file without including at least one -f filename.txt"
            )
            success = False
    return success


server_validators = {
    "apache": validate_apache_args,
    "nginx": validate_nginx_args,
    "iptables": validate_iptables_args,
    "ip6tables": validate_ip6tables_args,
}


def main():
    args = parse_args()
    if not validate_targets(args):
        raise SystemExit
    if args.servertype in server_validators:
        server_validators[args.servertype](args)
    build_date = datetime.now(timezone.utc)
    template_vars = {"header_comments": [], "ranges": []}
    for provider in args.targets:
        if args.asns and provider == "asn":
            provider_vars = get_ranges(
                provider, excludeip6=args.excludeip6, targets_in=args.asns
            )
        elif args.files and provider == "file":
            provider_vars = get_ranges(
                provider, excludeip6=args.excludeip6, targets_in=args.files
            )
        else:
            provider_vars = get_ranges(
                provider, excludeip6=args.excludeip6, compacted=args.compacted
            )
        template_vars["header_comments"] += provider_vars["header_comments"]
        template_vars["ranges"] += provider_vars["ranges"]
    template = get_template(args.servertype)
    template_output = build_template(
        template_vars, template, build_date, args.use_proxy, args.redir_target
    )
    outfile = get_output_path(args.servertype, args.targets, build_date)
    if not Path(output_dir).exists():
        Path(output_dir).mkdir()
    with open(outfile, "w") as o:
        o.write(template_output)

    print_output(args.servertype, args.targets, outfile)


if __name__ == "__main__":
    main()



import requests
from sephiroth.providers.base_provider import BaseProvider


class GCP(BaseProvider):
    def __init__(self, excludeip6=False):
        self.source_ranges = self._get_ranges()
        self.processed_ranges = self._process_ranges(excludeip6)

    def _get_ranges(self):
        """
        Input: None
        Output: Dict representation of cloud.json
        """
        print("(gcp) Fetching IP ranges from Google")
        gcp_ip_ranges_url = "https://www.gstatic.com/ipranges/cloud.json"
        r = requests.get(gcp_ip_ranges_url)
        return r.json()

    def _process_ranges(self, excludeip6=False):
        """
        Input: Dict of cloud.json, optionally exclude ip6 ranges
        Output: Dict with header_comments and list of dicts for ip ranges
        """
        header_comments = [
            f"(gcp) syncToken: {self.source_ranges['syncToken']}",
            f"(gcp) creationTime: {self.source_ranges['creationTime']}",
        ]
        out_ranges = []
        source_prefixes = self.source_ranges["prefixes"]

        for prefix in source_prefixes:
            if "ipv4Prefix" in prefix:
                item_prefix = prefix["ipv4Prefix"]
                iptype = "ipv4"
            elif "ipv6Prefix" in prefix and not excludeip6:
                item_prefix = prefix["ipv6Prefix"]
                iptype = "ipv6"

            item = {
                "range": item_prefix,
                "comment": f"{iptype} {prefix['scope']} {prefix['service']}",
            }
            out_ranges.append(item)

        return {"header_comments": header_comments, "ranges": out_ranges}



import os
from sephiroth.providers.base_provider import BaseProvider


class File(BaseProvider):
    def __init__(self, targets_in, excludeip6=False):
        self.source_ranges = self._get_ranges(targets_in)
        self.processed_ranges = self._process_ranges(excludeip6)

    def _get_ranges(self, target_files):
        """
        Input: List of filenames to read from
        Output: Dict representation of ip-ranges.json
        """
        ranges = {}
        print(f"(file) Reading IP ranges from {len(target_files)} files")
        for infile in target_files:
            with open(infile, "r") as f:
                ranges[os.path.basename(infile)] = f.readlines()
        return ranges

    def _process_ranges(self, excludeip6=False):
        """
        Input: Dict of ip-ranges.json, optionally exclude ip6 ranges
        Output: Dict with header_comments and list of dicts for ip ranges
        """
        header_comments = [
            f"(file) Ranges provided from {len(self.source_ranges.items())} input files"
        ]
        out_ranges = []
        for fname, range_list in self.source_ranges.items():
            for ip_line in range_list:
                if ip_line.startswith("#"):
                    continue
                if ":" in ip_line and excludeip6:
                    continue
                if "#" in ip_line:
                    ip_addr = ip_line.split("#")[0].strip()
                    comment = ip_line.split("#")[1].strip()
                else:
                    ip_addr = ip_line.strip()
                    comment = ""

                item = {"range": ip_addr, "comment": f"{fname} {comment}"}
                out_ranges.append(item)
        return {"header_comments": header_comments, "ranges": out_ranges}
